
#define _USE_MATH_DEFINES
#include <cmath>
#include <iostream>
#include <stdlib.h>


#include "rec/robotino/api2/all.h"

using namespace rec::robotino::api2;

float _rect[2]= {0,0};

bool _run = true;


class MyCom : public Com
{
public:
	MyCom()
		: Com( "rect" )
	{
	}

	void errorEvent( const char* errorString )
	{
		std::cerr << "Error: " << errorString << std::endl;
	}

	void connectedEvent()
	{
		std::cout << "Connected." << std::endl;
	}

	void connectionClosedEvent()
	{
		std::cout << "Connection closed." << std::endl;
	}

	void logEvent( const char* message, int level )
	{
		std::cout << message << std::endl;
	}

	void pingEvent( float timeMs )
	{
		//std::cout << "Ping: " << timeMs << "ms" << std::endl;
	}
};

class MyBumper : public Bumper
{
public:
	MyBumper()
		: bumped( false )
	{
	}

	void bumperEvent( bool hasContact )
	{
		bumped |= hasContact;
		std::cout << "Bumper has " << ( hasContact ? "contact" : "no contact") << std::endl;
	}

	bool bumped;
};


MyCom com;
OmniDrive omniDrive;
Odometry odometry;
MyBumper bumper;
CompactBHA cbha;


void init( const std::string& hostname )
{
	// Initialize the actors

	// Connect
	std::cout << "Connecting... ";
	com.setAddress( hostname.c_str() );

	com.connectToServer( true );

	if( false == com.isConnected() )
	{
		std::cout << std::endl << "Could not connect to " << com.address() << std::endl;

		rec::robotino::api2::shutdown();
		exit( 1 );
	}
	else
	{
		std::cout << "success" << std::endl;
        
        char x;
        
        
        cbha.setCompressorsEnabled( true );
        cbha.setGripperValve1( false );
        cbha.setGripperValve2( false );
        
        odometry.set(0,0,0,false);
        
	}
}
/*
void findVel(float &vx, float &vy, float&vphi, double x,double y,double phi){
    double ox,oy,ophi;
    odometry.readings(&ox,&oy,&ophi);
    vx=x/sqrt(x^2+y^2+phi^2);
    vy=y/sqrt(x^2+y^2+phi^2);
    vphi=phi/sqrt(x^2+y^2+phi^2);
}
*/
void drive()
{
    float targetValues[6] = { 0.591398, 0.522972, 0.532747, 0.796676, 0.821114, 0.904203};
    float caryPose[6] = { 0.591398, 0.522972, 0.532747, 0.796676, 0.821114, 0.904203};
    float pressures[8] = { 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f };
    float values[6] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
    float pickupPose[6]={0.777126,0.469208,0.454545,0.938416,0.782014,0.826002};
    float errors[6] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
    float tolerance=.02;
    
    
    float vel[3] = {0.0, 0.0, 0.0};
    unsigned int * seq;
    double x;
    double y;
    double phi;
    
    bool inTolerance=false;
    bool closed=false;
    
    const float maxPressure = 1.5f;
    float stepSize=.5;
    float lastTime=com.msecsElapsed();
    float lowerAvg;
    float upperAvg;

    for( unsigned int i = 0; i < 6; ++i ){
        targetValues[i]=pickupPose[i];
    }
	while( com.isConnected() && false == bumper.value() && _run )
	{
        // update the string pot values
        cbha.stringPots( values );
        
        // move toward target
        
        pressures[0]=pressures[0]+(-values[0]+targetValues[0])*stepSize+(values[1]-targetValues[1])*stepSize+(values[2]-targetValues[2])*stepSize;
        pressures[1]=pressures[1]+(values[0]-targetValues[0])*stepSize+(-values[1]+targetValues[1])*stepSize+(values[2]-targetValues[2])*stepSize;
        pressures[2]=pressures[2]+(values[0]-targetValues[0])*stepSize+(values[1]-targetValues[1])*stepSize+(-values[2]+targetValues[2])*stepSize;
        
        lowerAvg=(pressures[0]+pressures[1]+pressures[2])/3;
        
        pressures[0]-=lowerAvg;
        pressures[1]-=lowerAvg;
        pressures[2]-=lowerAvg;
            
        pressures[3]=pressures[3]+(-values[3]+targetValues[3])*stepSize+(values[4]-targetValues[4])*stepSize+(values[5]-targetValues[5])*stepSize;
        pressures[4]=pressures[4]+(values[3]-targetValues[3])*stepSize+(-values[4]+targetValues[4])*stepSize+(values[5]-targetValues[5])*stepSize;
        pressures[5]=pressures[5]+(values[3]-targetValues[3])*stepSize+(values[4]-targetValues[4])*stepSize+(-values[5]+targetValues[5])*stepSize;
 
        upperAvg=(pressures[3]+pressures[4]+pressures[5])/3;
        
        pressures[3]-=upperAvg;
        pressures[4]-=upperAvg;
        pressures[5]-=upperAvg;
        
        pressures[7]=10;
        
        cbha.setPressures( pressures );
        
        
        for( unsigned int i = 0; i < 6; ++i ){
            errors[i]=targetValues[i]-values[i];
            std::cout<< errors[i] << std::endl;
            if(tolerance > std::abs(errors[i])   ){
                
                std::cout << "in tolerance: " << i <<  std::endl;
            }
            else{
                break;
            }
            
            if(i==5){
                 inTolerance=true;
            }
            else inTolerance=false;
            
        }
        
        
        // movement
        
        odometry.readings( &x, &y, &phi );
        
        
        std::cout << "Pos:" << x << "," << y << ","<< phi << std::endl;
        
		omniDrive.setVelocity( vel[0], vel[1], vel[2] );

        
        // algorithm
        
        if (inTolerance==true && x<=0 && !closed){
            vel[0]=.2;
        }
        if (inTolerance==true && x<=0.51 && x>=.49 && !closed){
            vel[0]=0;
            rec::robotino::api2::msleep( 2000 );
            cbha.setGripperValve1( true );
            cbha.setGripperValve2( true );
            rec::robotino::api2::msleep( 1000 );
            closed=true;
            for( unsigned int i = 0; i < 6; ++i ){
                targetValues[i]=caryPose[i];
            }
        }
        if (inTolerance==true && x<=0.51 && x>=.49 && closed){
            vel[2]=.5;
        }
        if (phi>=2.2&&phi<=2.3) {
            vel[2]=0;
            vel[0]=.2;
            rec::robotino::api2::msleep( 5000 );
            break;
        }
        
        
        
        // list values
        
        std::cout << std::endl;
        
        if (inTolerance){
         std::cout << " all in tolerance: "<< std::endl;
        }
        std::cout << std::endl;
        
        for( unsigned int i = 0; i < 8; ++i )
        {
            std::cout << "B" << i << ": " << pressures[i] << " bar" << std::endl;
        }
        
        std::cout << std::endl;
        
        for( unsigned int i = 0; i < 6; ++i )
        {
            std::cout << "String pot " << i  << ": " << values[i] << std::endl;
        }

        
        // syncronize and delay
        lastTime=com.msecsElapsed();
		com.processEvents();
		rec::robotino::api2::msleep( 100 );
	}
    
    
}

void destroy()
{
	com.disconnectFromServer();
}

int main( int argc, char **argv )
{

	std::string hostname = "172.26.1.1";
 
	if( argc > 1 )
	{
		hostname = argv[1];
	}

	try
	{
		init( hostname );
		drive();
		destroy();
	}
	catch( const rec::robotino::api2::RobotinoException& e )
	{
		std::cerr << "Com Error: " << e.what() << std::endl;
	}
	catch( const std::exception& e )
	{
		std::cerr << "Error: " << e.what() << std::endl;
	}
	catch( ... )
	{
		std::cerr << "Unknow Error" << std::endl;
	}

	rec::robotino::api2::shutdown();
}